package syncloud.filesystem;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import syncloud.core.log.Logger;
import syncloud.storage.*;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

public class FSFile implements IFile {
    private NodeKey key;
    private File file;
    private static Logger logger = Logger.getLogger(FSFile.class);

    /*public FSFile(NodeKey key, File file) {
        this.file = file;
        this.key = key;
    }*/

    public FSFile(NodeKey key) {
        this.key = key;
        this.file = new File(key.getNativePath());
    }

    @Override
    public NodeKey getKey() {
        return key;
    }

    @Override
    public boolean exists() throws StorageException {
        return file.exists();
    }


    @Override
    public void delete() throws StorageException {
        try {
            boolean deleted = file.delete();
            if (!deleted) throw new IOException();
        } catch (Throwable e) {
            String message = String.format(UNABLE_TO_DELETE_FILE, key.getNativePath());
            logger.error(message);
            throw new StorageException(message);
        }
    }

    @Override
    public String getName() {
        return file.getName();
    }

    @Override
    public InputStream getData() throws StorageException {
        try {
            return FileUtils.openInputStream(file);
        } catch (IOException e) {
            String message = UNABLE_TO_GET_DATA_FOR_FILE + file.getName();
            logger.error(message, e);
            throw new StorageException(message);
        }
    }

    @Override
    public String getVersion() throws StorageException {
        if (!exists()) return null;

        InputStream fis = null;
        try {
            fis = getData();
            return new String(Hex.encodeHex(DigestUtils.md5(fis)));
        } catch (Throwable e) {
            String message = String.format(UNABLE_TO_GET_VERSION_OF_FILE, file.getName());
            logger.error(message, e);
            throw new StorageException(message, e);
        } finally {
            if (fis != null) try {
                fis.close();
            } catch (IOException e) {
                logger.error("unable to close input stream, file: " + file.getName());
            }
        }
    }

    @Override
    public void save(InputStreamProvider inputStreamProvider, long size) throws StorageException {
        FileOutputStream outputStream = null;
        try {
            outputStream = new FileOutputStream(file, false);
            IOUtils.copy(inputStreamProvider.getData(), outputStream);
        } catch (IOException e) {
            String message = UNABLE_TO_WRITE_DATA_TO_FILE + file.getName();
            logger.error(message, e);
            throw new StorageException(message);
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    String message = UNABLE_TO_WRITE_DATA_TO_FILE + file.getName();
                    logger.error(message, e);
                    throw new StorageException(message);
                }
            }
        }
    }

    @Override
    public long size() {
        return file.length();
    }

    public void createNewFile() {
        try {
            boolean ignore = file.createNewFile();
        } catch (IOException e) {
            String message = IFolder.UNABLE_TO_CREATE_FILE(file.getName());
            logger.error(message, e);
            throw new StorageException(message);
        }
    }
}
